<?php

namespace FreshMail;

use Configuration;
use FreshMail\Entity\Cart;
use FreshMail\Repository\AsyncJobs;
use FreshMail\Repository\FormRepository;
use FreshMail\Repository\FreshmailAbandonCartSettings;
use FreshMail\Repository\FreshmailSettings;
use FreshMail\Service\FormService;
use Validate;

trait Hooks
{

    public function getHooks()
    {
        return [
            'displayBackOfficeHeader',
            'displayTop',
            'displayLeftColumnProduct',
            'displayRightColumnProduct',
            'displayFooterProduct',
            'actionCustomerAccountAdd',
            'actionDeleteGDPRCustomer',
            'actionObjectCustomerDeleteAfter',
            'actionObjectCartAddAfter'
        ];
    }

    public function hookDisplayBackOfficeHeader($params)
    {
        $this->context->controller->addCSS($this->_path . 'views/css/freshmail-core.css', 'all');

        $ets = new \FreshMail\Repository\EmailToSynchronize(\Db::getInstance());
        $list = $ets->getListToSync();
        if(!empty($list[0])){
            $this->context->smarty->assign([
                'pendingSend' => true,
                'sendUrl' => $this->context->link->getBaseLink().'modules/'.$this->name.'/cron/send_subscribers.php?hash='.$list[0]['hash_list'].'&token='.$this->getCronToken()
            ]);
        }

        $aj = new AsyncJobs(\Db::getInstance());
        $jobs = $aj->getRunningJobs();
        if(!empty($jobs)){
            Tools::asyncJobPing();
        }

        return $this->display(_PS_MODULE_DIR_ . $this->name, '../admin/header.tpl');
    }


    public function hookDisplayTop($params)
    {
        $formService = new FormService(new FormRepository());
        $fields = $formService->getActiveFormByPosition(FormRepository::$TOP);

        return $this->initForm('top', $fields);
    }

    public function hookDisplayLeftColumnProduct()
    {
        $formService = new FormService(new FormRepository());
        $fields = $formService->getActiveFormByPosition(FormRepository::$LEFT_COLUMN);
        return $this->initForm('left', $fields);
    }

    public function hookDisplayRightColumnProduct()
    {
        $formService = new FormService(new FormRepository());
        $fields = $formService->getActiveFormByPosition(FormRepository::$RIGHT_COLUMN);

        return $this->initForm('right', $fields);
    }

    public function hookDisplayFooterProduct()
    {
        $formService = new FormService(new FormRepository());
        $fields = $formService->getActiveFormByPosition(FormRepository::$FOOTER);
        return $this->initForm('footer', $fields);
    }

    private function initForm($position, $fields)
    {
        if (!$fields) {
            return false;
        }

        $formName = $fields[0]['form_name'];
        $subscribersStatus = $fields[0]['subscribers_status'];

        $position = mb_strtolower($position);

        $forms = array();
        foreach ($fields as $k => $field) {
            $forms[$field['id_form']][] = $field;
        }

        if (count($forms) == 0) {
            return false;
        }
        $msg = [];
        foreach ($forms as $id => $v) {
            if (isset($_POST['submit_freshmail_' . $id])) {
                $msg = $this->_submit($subscribersStatus);
            }
        }


        $this->context->controller->addCSS($this->_path . 'view/css/front_' . $position . '.css', 'all');
        $data = [
            'placement' => $position,
            'forms' => $forms,
            'form_name' => $formName,
            'error' => isset($msg['error']) ? $msg['error'] : '',
            'success' => isset($msg['success']) ? $msg['success'] : '',
            'label_submit' => Configuration::get('FRESHMAIL_FORM_SUBSCRIBE_BUTTON')
        ];

        $this->context->smarty->assign($data);
        return $this->display(_PS_MODULE_DIR_ . $this->name, $position . '.tpl');
    }

    private function _submit($subscribersStatus = 0)
    {
        if (!empty($_POST)) {
            $freshmailSettings = (new FreshmailSettings())->findForShop($this->context->shop->id);

            $this->displayConfirmation($this->l('Settings updated'));
            $list = $freshmailSettings->subscriber_list_hash;

            $tags = \Tools::getValue('tag');
            $email = isset($tags['email']) ? $tags['email'] : $this->context->customer->email;

            if (!Validate::isEmail($email)) {
                $this->context->smarty->assign(array(
                    'error' => Configuration::get('FRESHMAIL_INVALID_EMAIL_ADDRESS_MESSAGE'),
                ));
            } elseif (empty($tags)) {
                $this->context->smarty->assign(array(
                    'error' => Configuration::get('FRESHMAIL_REQUIRED_FIELD_MISSING_MESSAGE'),
                ));
            } else {
                $data = array(
                    'email' => $email,
                    'list' => $list,
                    'custom_fields' => array(),
                    'state' => $subscribersStatus
                );
                unset($tags['email']);

                if (!empty($tags)) {
                    foreach ($tags as $k => $v) {
                        $data['custom_fields'][$k] = $v;
                    }
                }
                $freshmail = new Freshmail($freshmailSettings->api_token);
                return $freshmail->addSubscriber($data);
            }
        }
    }

    public function hookActionCustomerAccountAdd($params){
        $customer = $params['newCustomer'];

        if($customer->newsletter){
            $freshmailSettings = (new FreshmailSettings())->findForShop($this->context->shop->id);
            $fmList = $this->getFreshmailList();
            if(empty($fmList)){
                return;
            }

            $subscriber = new Subscriber($customer->email);
            $subscriber->custom_fields[\Freshmail::NAME_TAG] = $customer->firstname ;
            $subscriber->state = 0 == $freshmailSettings->send_confirmation? 1 : 0;
            $fmList->addSubscriber($subscriber);
        }
    }

    public function hookActionDeleteGDPRCustomer($customer)
    {
        if (!empty($customer['email']) && Validate::isEmail($customer['email'])) {
            $fmList = $this->getFreshmailList();
            if(empty($fmList)){
                return;
            }
            $fmList->deleteSubscriber(new Subscriber($customer['email']));
        }
    }

    public function hookActionObjectCustomerDeleteAfter($params)
    {
        $email = $params['object']->email;
        $fmList = $this->getFreshmailList();

        if(empty($fmList)){
            return;
        }
        $fmList->deleteSubscriber(new Subscriber($email));
    }


    public function getFreshmailList() : FreshmailList
    {
        $freshmailSettings = (new FreshmailSettings())->findForShop($this->context->shop->id);
        $fm = new FreshmailList($freshmailSettings->api_token, $freshmailSettings->subscriber_list_hash);
        if(empty($freshmailSettings->subscriber_list_hash) || !$fm->check()){
            return false;
        }

        return $fm;
    }

    public function hookActionObjectCartAddAfter($params)
    {
        $cart = $params['object'];
        if(
            !(new FreshmailAbandonCartSettings(\Db::getInstance()))->findForShop($cart->id_shop)->enabled
        ){
            return;
        }

        $fmCart = new Cart();
        $fmCart->id_cart = $cart->id;
        $fmCart->cart_token = sha1(time()).md5(time());
        $fmCart->save();
    }
}