<?php

namespace FreshMail\Sender;

use FreshMail\Entity\Cart;
use FreshMail\Freshmail;
use FreshMail\FreshmailApiV3;
use FreshMail\Interfaces\Sender;
use FreshMail\Repository\FreshmailSettings;
use FreshMail\TransactionalEmail;

class FmSender extends AbstractSender implements Sender{

    public function send(\Cart $cart, Cart $fmCart, Email $email) : bool
    {
        $shop = new \Shop($cart->id_shop);

        $context = \Context::getContext();
        $context->cart = $cart;
        $context->shop = new \Shop($cart->id_shop);

        $customer = new \Customer($cart->id_customer);
        if(empty($customer->email)){
            return false;
        }


        $html = str_replace(
            ['{products_list}', '{discount_code}', '{shop_name}', '{shop_url}', '{cart_url}'],
            [AbstractSender::getProductList($cart, $context), AbstractSender::getDiscountCode($fmCart), $shop->name, $context->shop->getBaseURL(), AbstractSender::getCartUrl($fmCart)],
            $this->getMailHtml($cart, $fmCart)
        );

        $sender = new Email(\Configuration::get('PS_SHOP_EMAIL'), $shop->name);


        $fmApi = new FreshmailApiV3($this->fmSettings->api_token);
        $cs = $this->cartSettings->findForShop($cart->id_shop);

        return $fmApi->sendTransactionalEmail(
            new TransactionalEmail($email, $sender, $cs->email_subject[$cart->id_lang], $html)
        );
    }

    private function getMailHtml(\Cart $cart, Cart $fmCart) : string
    {
        $cs = $this->cartSettings->findForShop($cart->id_shop);
        if(!empty($cs->template_id_hash)){
            $fm = new Freshmail($this->fmSettings->api_token);
            $html = $fm->getTemplateHtml($cs->template_id_hash);

            if(!empty($html)){
                return $html;
            }
        }


        $lang = new \Language($cart->id_lang);
        return file_get_contents(_PS_MODULE_DIR_.'freshmail/mails/'.$lang->iso_code.'/abandoned-cart.html');
    }
}