<?php

use FreshMail\Entity\Form;
use FreshMail\Service\FieldService;
use FreshMail\Service\FormService;

require_once __DIR__ . '/AdminFreshmailBase.php';

class AdminFreshmailFormConfigController extends AdminFreshmailBaseController
{


    public static $UPDATE_SUCCESS = "Success!";
    public static $ERROR = "Error!";
    public static $POSITION = array(
        '1' => 'Displays new elements in the left-hand column of the product page',
        '2' => 'Displays new elements in the right-hand column of the product page',
        '3' => 'Added under the product’s description',
        '4' => 'Top of pages This hook displays additional elements at the top of your pages'
    );

    public function initContent()
    {


        /*  ($this->get('FreshMail.Service.Auth'))::check();
          $this->api = $this->get('FreshMail.Service.Api');
          $this->api->setAccess(Configuration::get('FRESHMAIL_API_KEY'), Configuration::get('FRESHMAIL_API_SECRECT_KEY'));*/

        $this->initTabModuleList();
        $this->initToolbar();


        parent::initContent();

    }

    public function initPageHeaderToolbar()
    {
        if (empty($this->display)) {
            $this->page_header_toolbar_btn['new_product'] = array(
                'href' => self::$currentIndex . '&add' . $this->table . '=1&token=' . $this->token,
                'desc' => $this->module->l('Add new form'),
                'icon' => 'process-icon-new'
            );
        }

        parent::initPageHeaderToolbar();
    }

    public function initToolbar()
    {
        switch ($this->display) {
            default: // list
                $this->toolbar_btn['new'] = array(
                    'href' => self::$currentIndex . '&add' . $this->table . '=1&token=' . $this->token,
                    'desc' => $this->module->l('Add New Form')
                );
        }

        unset($this->toolbar_btn['back']);
    }

    public function __construct()
    {

        $this->bootstrap = true;
        $this->table = 'freshmail_form';
        $this->tpl_folder = 'freshmail_controller';
        $this->lang = false;
        $this->explicitSelect = true;
        $this->context = Context::getContext();
        $this->show_cancel_button = false;


        parent::__construct();
    }


    public function renderList()
    {
        if(empty($this->freshmailSettings->api_token)){
            $this->errors[] = $this->module->l('To use this feature You have to enable synchronization with FreshMail');
            return;
        }

        $this->fields_list = array(
            'id_freshmail_form' => array(
                'title' => 'ID',
                'align' => 'center',
                'width' => 25
            ),
            'name' => array(
                'title' => $this->module->l('Name'),
                'width' => 'auto',
            ),
            'hash_list' => array(
                'title' => $this->module->l('List Hash'),
                'width' => 'auto'
            ),
            'position' => array(
                'title' => $this->module->l('Position'),
                'width' => 'auto'
            ),
            'count_fields' => array(
                'title' => $this->module->l('Count fields (include)'),
                'width' => 'auto'
            ),
            'active' => array(
                'title' => $this->module->l('Active'),
                'width' => 'auto'
            ),
        );

        //    $hashListSQL = $this->api->getAllHashListSQL();
        $hashList = $this->freshmail->getAllHashList();

        /** @var FormService $formService */
        $formService = new FormService(new \FreshMail\Repository\FormRepository());
        $forms = $formService->getAllFormsFull();

        foreach ($forms as $k => $form) {
            if (!in_array($form['hash_list'], $hashList)) {
                $formObj = new FreshMail\Entity\Form($form['id_freshmail_form']);
                $formObj->active = 0;
                $formObj->update(true);
            }
        }

        $this->_select = 'a.`id_freshmail_form`, a.`hash_list`, a.`position`, a.`active`, (select count(id) from ' . _DB_PREFIX_ . 'freshmail_list_field where id_form = a.`id_freshmail_form` and include_field = 1) as `count_fields`, `a`.`name`';
        $this->_group = 'GROUP BY a.id_freshmail_form';
        $this->_orderBy = 'id_freshmail_form';
        $this->_orderWay = 'DESC';


        // Adds an Edit button for each result
        $this->addRowAction('edit');

        // Adds a Delete button for each result
        $this->addRowAction('delete');
        $this->specificConfirmDelete = $this->l('Delete selected items?', array(), 'Admin.Notifications.Warning');


        if (!($this->fields_list && is_array($this->fields_list))) {
            return false;
        }
        $this->getList($this->context->language->id);

        foreach ($this->_list as $k => $v) {
            if (isset($v['active'])) {
                if ((int)$this->_list[$k]['active'] == 0) {
                    $this->_list[$k]['active'] = $this->module->l('Disabled');
                } else {
                    $this->_list[$k]['active'] = $this->module->l('Enabled');
                }

                if (isset($v['position']) && isset(self::$POSITION[$this->_list[$k]['position']])) {
                    $this->_list[$k]['position'] = $this->module->l(self::$POSITION[$this->_list[$k]['position']]);
                }
            }
        }



        // If list has 'active' field, we automatically create bulk action
        if (isset($this->fields_list) && is_array($this->fields_list) && array_key_exists('active', $this->fields_list)
            && !empty($this->fields_list['active'])) {
            if (!is_array($this->bulk_actions)) {
                $this->bulk_actions = array();
            }
        }

        $helper = new HelperList();

        // Empty list is ok
        if (!is_array($this->_list)) {
            $this->displayWarning($this->module->l('Bad SQL query', 'Helper') . '<br />' . htmlspecialchars($this->_list_error));
            return false;
        }

        $this->setHelperDisplay($helper);
        $helper->simple_header = true;
        $helper->_default_pagination = $this->_default_pagination;
        $helper->_pagination = $this->_pagination;
        $helper->tpl_vars = $this->getTemplateListVars();

        // For compatibility reasons, we have to check standard actions in class attributes
        foreach ($this->actions_available as $action) {
            if (!in_array($action, $this->actions) && isset($this->$action) && $this->$action) {
                $this->actions[] = $action;
            }
        }
        $helper->has_value = false;
        $helper->is_cms = $this->is_cms;
        $helper->sql = $this->_listsql;
        $list = $helper->generateList($this->_list, $this->fields_list);

        return $list;
    }

    private function valid($action = 'add')
    {
        $this->requiredFields = array('name', 'position');
        foreach ($this->requiredFields as $k => $v) {
            if (!empty($v)) {
                $value = Tools::getValue($v);
                if (empty($value)) {
                    $title = ucfirst($this->fields_list[$v]['title']);
                    $label = $title . $this->module->l(' is required!') . ' -> ' . $v;
                    $this->errors[] = Tools::displayError($label);
                }
            }
        }
        if (is_array($this->errors) && count($this->errors)) {
            $this->action = $action;
            return false;
        }
        return true;
    }


    public function postProcess()
    {
        $_POST['hash_list'] = $this->freshmailSettings->subscriber_list_hash;

        if (Tools::getValue('addid_freshmail_form')) {
            $action = 'addfreshmail_form';
        } elseif (Tools::getValue('editid_freshmail_form')) {
            $action = 'updatefreshmail_form';
        }

        if (Tools::isSubmit('submitFieldsList') && Tools::getValue('addid_freshmail_form')) {


            if (!$this->valid('add')) {
                return parent::postProcess();
            }
            try {
                $newForm = new Form();
                $newForm->name = Tools::getValue('name');
                $newForm->hash_list = $this->freshmailSettings->subscriber_list_hash;
                $newForm->position = Tools::getValue('position');
                $newForm->subscribers_status = Tools::getValue('subscribers_status');
                $newForm->active = Tools::getValue('active');
                $newForm->add();
                $fieldsList = Tools::getValue('fields');
                if (empty($fieldsList)) {
                    $fieldsList = array();
                }

                foreach ($fieldsList as $hashList => $fields) {
                    $field = new FreshMail\Entity\Field();

                    // $field->hash_list = $this->freshmailSettings->subscriber_list_hash;
                    $field->name = $fields['name'];
                    $field->tag = $fields['tag'];
                    $field->hash = $fields['hash'];
                    $field->type = $fields['type'];
                    $field->displayname = $fields['displayname'];

                    if (isset($fields['require_field'])) {
                        $field->require_field = true;
                    }
                    if (isset($fields['include_field'])) {
                        $field->include_field = true;
                    }

                    $field->id_form = $newForm->id;

                    $id = $field->save();
                }
                $link = $this->context->link->getAdminLink('AdminFreshmailFormConfig') . '&conf=4&updatefreshmail_form=1&id_freshmail_form=' . $newForm->id;
                Tools::redirectAdmin($link);
            } catch (Exception $e) {
                if (!is_array($this->errors) || count($this->errors) <= 0) {
                    $this->errors[] = Tools::displayError($e->getMessage());
                }
            }
        } elseif (Tools::isSubmit('submitFieldsList') && Tools::getValue('editid_freshmail_form')) {

            try {
                $id = (int)Tools::getValue('id_freshmail_form');

                $form = new FreshMail\Entity\Form($id);
                $form->name = Tools::getValue('name');
                $form->position = Tools::getValue('position');
                $form->subscribers_status = Tools::getValue('subscribers_status');
                $form->active = Tools::getValue('active');
                $form->update();

                $fieldsList = Tools::getValue('fields');

                foreach ($fieldsList as $hashList => $fields) {
                    $field = new FreshMail\Entity\Field($fields['id']);
                    $field->displayname = $fields['displayname'];
                    $field->hash = $fields['hash'];
                    $field->require_field = isset($fields['require_field']) ? 1 : 0;
                    $field->include_field = isset($fields['include_field']) ? 1 : 0;
                    $field->name = $fields['name'];
                    $field->tag = $fields['tag'];
                    $field->id_form = $form->id;
                    $field->type = $fields['type'];
                    $field->save();
                }

                $link = $this->context->link->getAdminLink('AdminFreshmailFormConfig', true) . '&conf=4&updatefreshmail_form=1&id_freshmail_form=' . $id;
                Tools::redirectAdmin($link);
            } catch (Exception $e) {
                $this->displayWarning($this->module->l(self::$ERROR) . ' ' . $e->getMessage());
            }
        }
        return parent::postProcess();
    }

    // This method generates the Add/Edit form
    public function renderForm()
    {


        $this->submit_action = $this->display . $this->identifier;
        $this->show_form_cancel_button = false;

        $lists = $this->freshmail->getLists();

        $this->fields_form = array(
            'legend' => array(
                'title' => $this->module->l('Forms and additional fields'),
            ),
            'input' => array(
                array(
                    'type' => 'hidden',
                    'name' => 'id_freshmail_form'
                ),
                array(
                    'type' => 'text',
                    'label' => $this->module->l('Name'),
                    'name' => 'name',
                    'required' => true,
                    'desc' => $this->module->l('Form name'),
                    'class' => 'fixed-width-xxl',
                    'hint' => $this->module->l('Invalid characters:') . ' &lt;&gt;;=#{}',
                ),
                array(
                    'type' => 'hidden',
                    'label' => $this->module->l('List'),
                    'desc' => $this->module->l('Lists of recipients defined in the Freshmail application'),
                    'name' => 'hash_list',
                    'multiple' => false,
                    'required' => true,
                    'options' => array(
                        'query' => $lists,
                        'id' => 'subscriberListHash',
                        'name' => 'name'
                    ),
                    "disabled" => true
                ),
                array(
                    'type' => 'select',
                    'label' => $this->module->l('Position'),
                    'name' => 'position',
                    'required' => true,
                    'options' => array(
                        'query' => array(
                            array('key' => '1', 'name' => $this->module->l('Displays new elements in the left-hand column of the product page')),
                            array('key' => '2', 'name' => $this->module->l('Displays new elements in the right-hand column of the product page')),
                            array('key' => '3', 'name' => $this->module->l('Added under the product’s description')),
                            array('key' => '4', 'name' => $this->module->l('Top of pages This hook displays additional elements at the top of your pages')),
                        ),
                        'id' => 'key',
                        'name' => 'name'
                    ),
                    'desc' => $this->module->l('For Left or Rigth product column, first Go To PrestaShop Admin->Design->Theme & Logo > Theme Full Width - No side columns, ideal for distraction-free pages such as product pages" ->assign layout to value "Two Columns, small right column - Two columns with a small right column".')
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->module->l('Enable form'),
                    'name' => 'active',
                    'required' => true,
                    'class' => 'switch prestashop-switch',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->module->l('Enabled')
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->module->l('Disabled')
                        )
                    ),
                ),
                array(
                    'type' => 'select',
                    'label' => $this->module->l('Status to be added to subscriber.'),
                    'name' => 'subscribers_status',
                    'required' => true,
                    'value' => 0,
                    'options' => array(
                        'query' => array(
                            array('key' => '1', 'name' => $this->module->l('ACTIVE')),
                            array('key' => '0', 'name' => $this->module->l('WAITING'))
                        ),
                        'id' => 'key',
                        'name' => 'name',
                    ),
                ),
                array(
                    'type' => 'hidden',
                    'name' => 'cal_field_name',
                    'value' => $this->module->l('Field name')
                ),
                array(
                    'type' => 'hidden',
                    'name' => 'cal_tag_name',
                    'value' => $this->module->l('Tag name')
                ),
                array(
                    'type' => 'hidden',
                    'name' => 'cal_text_on_the_form',
                    'value' => $this->module->l('Text on the form')
                ),
                array(
                    'type' => 'hidden',
                    'name' => 'cal_required',
                    'value' => $this->module->l('Required')
                ),
                array(
                    'type' => 'hidden',
                    'name' => 'cal_include',
                    'value' => $this->module->l('Include')
                ),
            ),
            'submit' => array(
                'name' => 'submitFieldsList',
                'title' => $this->module->l('Save'),
                'class' => 'btn btn-default pull-right'
            ),
            'buttons' => array(
                array(
                    'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminFreshmailFormConfig'),
                    'title' => $this->module->l('Back to list'),
                    'icon' => 'process-icon-back'
                )
            )
        );

        $form = new FreshMail\Entity\Form((int)Tools::getValue('id_freshmail_form'));

        $this->fields_value['name'] = $form->name;
        $this->fields_value['position'] = $form->position;
        $this->fields_value['hash_list'] = $form->hash_list;
        $this->fields_value['active'] = $form->active;
        $this->fields_value['confirm'] = $form->confirm;
        $this->fields_value['subscribers_status'] = $form->subscribers_status;
        $this->fields_value['cal_field_name'] = $this->module->l('Field name');
        $this->fields_value['cal_tag_name'] = $this->module->l('Tag name');
        $this->fields_value['cal_text_on_the_form'] = $this->module->l('Text on the form');
        $this->fields_value['cal_required'] = $this->module->l('Required');
        $this->fields_value['cal_include'] = $this->module->l('Include');

        $fieldsDb = $form->fields;
        // LK todo
        $fieldsApi = []; //$this->api->getAllFieldsByIdHashList($form->hash_list);
        $this->refreshFields($fieldsApi, $fieldsDb, $form->hash_list, $form->id);
        $fieldsList = false;

        if ($fieldsDb) {
            $fieldsList = $fieldsDb;
        }


        $this->context->smarty->assign(
            array(
                'fields_list' => true,
                'fields_list_update' => ($fieldsList) ? $fieldsList : false,
                'id' => $form->id,
                'display' => $this->display
            )
        );

        return parent::renderForm();
    }

    public function refreshFields($fieldsApi = null, $fieldsDb = null, $hashList = null, $formId = null)
    {
        if ((empty($fieldsApi) && empty($fieldsDb)) || !$hashList || !$formId) {
            return false;
        }

        $hashKeys = $this->freshmail->getAllFieldHashByHashList($hashList);

        $fieldService = new FieldService(new \FreshMail\Repository\FieldRepository());

        $fieldModel = new FreshMail\Entity\Field();
        /** @var FieldService $fieldService */
        $hashListFieldDb = $fieldService->getFieldsHashByIdForm($formId);
        $change = 0;
        $delete = 0;

        foreach ($fieldsDb as $k => $field) {
            if ($field['tag'] != 'email' && !in_array($field['hash'], array_keys($hashKeys))) {
                $fieldObj = new FreshMail\Entity\Field($field['id']);
                $fieldObj->delete();
                $delete = 1;
            } else if ($field['tag'] != 'email' && ($hashKeys[$field['hash']]['tag'] != $field['tag'] || $hashKeys[$field['hash']]['type'] != $field['type'])) {
                $fieldObj = new FreshMail\Entity\Field($field['id']);
                $fieldObj->tag = $hashKeys[$field['hash']]['tag'];
                $fieldObj->type = $hashKeys[$field['hash']]['type'];
                $fieldObj->update();
                $change = 1;
            }
        }

        foreach ($fieldsApi as $k => $field) {
            if (!in_array($field['hash'], $hashListFieldDb)) {
                $fieldObj = new FreshMail\Entity\Field();
                foreach ($field as $fieldName => $fieldValue) {
                    if (!empty($fieldValue)) {
                        $fieldObj->{$fieldName} = $fieldValue;
                    }
                }
                $fieldObj->hash_list = $hashList;
                $fieldObj->id_form = $formId;
                $fieldObj->save();
                $change = 1;
            }
        }

        if ($change || $delete) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminFreshmailFormConfig') . '&id_freshmail_form=' . (int)$formId . '&updatefreshmail_form=1&conf=4');
        }
    }

    public function processFilter()
    {

    }

    public function displayEditLink($token = null, $id, $name = null)
    {
        $tpl = $this->createTemplate('helpers/list/list_action_edit.tpl');
        if (!array_key_exists('Edit', self::$cache_lang)) {
            self::$cache_lang['Edit'] = $this->module->l('Edit', 'Helper');
        }

        $href = self::$currentIndex . '&' . $this->identifier . '=' . $id . '&update' . $this->table . '&token=' . ($token != null ? $token : $this->token);

        if ($this->display == 'view') {
            $href = Context::getContext()->link->getAdminLink('AdminCustomers') . '&id_customer=' . (int)$id . '&updatecustomer';
        }

        $tpl->assign(array(
            'href' => $href,
            'action' => self::$cache_lang['Edit'],
            'id' => $id
        ));

        return $tpl->fetch();
    }

    public function checkAccess($disable = false)
    {
        return true;
    }

    public function viewAccess($disable = false)
    {
        return true;
    }

    public function displayAjax()
    {
        $return = array(
            'hasError' => true,
            'errors' => 'Error'
        );
        die(Tools::jsonEncode($return));
    }

    public function setMedia($isNewTheme = false)
    {
        parent::setMedia($isNewTheme);
        $this->addJS(
            array(
                _PS_MODULE_DIR_ . $this->module->name . '/views/js/updateFieldsForm.js?'.$this->module->version
            )
        );

    }

    public function displayAjaxUpdateFieldsForm()
    {
        $this->ajax = true;
        $hash = Tools::getValue('hash');
        $fieldService = new FieldService(new \FreshMail\Repository\FieldRepository());

        $fieldsApiArray = $this->freshmail->getAllFieldsByIdHashList($hash);

        $idForm = Tools::getValue('id_form');
        $fieldsDbArray = !empty($idForm) ? $fieldService->getFieldsByIdForm($idForm) : [];


        $fields = [];
        foreach ($fieldsDbArray as $key => $value) {
            $fields[$value['hash']] = [
                'id' => $value['id'],
                'displayname' => $value['displayname'],
                'hash' => $value['hash'],
                'name' => $value['name'],
                'tag' => $value['tag'],
                'require_field' => $value['require_field'],
                'include_field' => $value['include_field']
            ];
        }

        foreach ($fieldsApiArray as $key => $value) {
            if (!isset($fields[$value['hash']])) {
                $fields[$value['hash']] = $fieldsApiArray[$key];
            }
        }

        if (!$this->checkExistField('email', $fields)) {
            $fields[] = array(
                'displayname' => "E-mail",
                'hash' => $hash,
                'name' => "E-mail",
                'tag' => "email",
                'require_field' => 1,
                'include_field' => 1
            );
        }

        $result = array(
            'status' => 'OK',
            'fields' => $fields
        );
        echo Tools::jsonEncode($result);
    }

    private function checkExistField($fieldName, $fields)
    {
        foreach ($fields as $key => $value) {
            if ($fieldName == $value['tag']) {
                return true;
            }
        }
        return false;
    }

    public function processDelete()
    {
        $id = (int)Tools::getValue('id_freshmail_form');

        $form = new FreshMail\Entity\Form($id);
        $form->delete();
        Tools::redirectAdmin($this->context->link->getAdminLink('AdminFreshmailFormConfig') . '&conf=1');
    }

    public function processAdd()
    {
        $this->display = 'add';
    }
}
